@@ -129,11 +129,16 @@
  * unless the user has a _really_ huge screen.
  */
 
+#include <config.h>
+#include "gdkalias.h"
 #include "gdk.h"		/* For gdk_rectangle_intersect */
 #include "gdkprivate-x11.h"
 #include "gdkx.h"
 #include "gdkregion.h"
 #include "gdkinternals.h"
+#include "gdkscreen-x11.h"
+#include "gdkdisplay-x11.h"
+#include "gdkwindow-x11.h"
 
 typedef struct _GdkWindowQueueItem GdkWindowQueueItem;
 typedef struct _GdkWindowParentPos GdkWindowParentPos;
@@ -180,14 +185,10 @@ static void gdk_window_postmove           (GdkWindow          *window,
 static void gdk_window_queue_translation  (GdkWindow          *window,
 					   gint                dx,
 					   gint                dy);
-static void gdk_window_tmp_unset_bg       (GdkWindow          *window);
-static void gdk_window_tmp_reset_bg       (GdkWindow          *window);
 static void gdk_window_clip_changed       (GdkWindow          *window,
 					   GdkRectangle       *old_clip,
 					   GdkRectangle       *new_clip);
 
-static GQueue *translate_queue = NULL;
-
 void
 _gdk_windowing_window_get_offsets (GdkWindow *window,
 				   gint      *x_offset,
@@ -226,25 +227,19 @@ gdk_window_copy_area_scroll (GdkWindow    *window,
 
   if (dest_rect->width > 0 && dest_rect->height > 0)
     {
-      GC gc;
-      XGCValues values;
-
-      values.graphics_exposures = True;
-      gc = XCreateGC (GDK_WINDOW_XDISPLAY (window),
-		      GDK_WINDOW_XID (window),
-		      GCGraphicsExposures, &values);
+      GdkGC *gc;
 
+      gc = _gdk_drawable_get_scratch_gc (window, TRUE);
+      
       gdk_window_queue_translation (window, dx, dy);
 
       XCopyArea (GDK_WINDOW_XDISPLAY (window),
 		 GDK_WINDOW_XID (window),
 		 GDK_WINDOW_XID (window),
-		 gc,
+		 gdk_x11_gc_get_xgc (gc),
 		 dest_rect->x - dx, dest_rect->y - dy,
 		 dest_rect->width, dest_rect->height,
 		 dest_rect->x, dest_rect->y);
-
-      XFreeGC (GDK_WINDOW_XDISPLAY (window), gc);
     }
 
   tmp_list = obj->children;
@@ -329,10 +324,10 @@ gdk_window_guffaw_scroll (GdkWindow    *window,
   parent_pos.x11_y += new_info.y;
   parent_pos.clip_rect = new_info.clip_rect;
 
-  gdk_window_tmp_unset_bg (window);
+  _gdk_x11_window_tmp_unset_bg (window, FALSE);;
 
-  if (d_xoffset < 0 || d_yoffset < 0)
-    gdk_window_queue_translation (window, MIN (d_xoffset, 0), MIN (d_yoffset, 0));
+  if (dx > 0 || dy > 0)
+    gdk_window_queue_translation (window, MAX (dx, 0), MAX (dy, 0));
 	
   gdk_window_set_static_gravities (window, TRUE);
 
@@ -357,8 +352,8 @@ gdk_window_guffaw_scroll (GdkWindow    *window,
 	       GDK_WINDOW_XID (window),
 	       new_position.x - d_xoffset, new_position.y - d_yoffset);
   
-  if (d_xoffset > 0 || d_yoffset > 0)
-    gdk_window_queue_translation (window, MAX (d_xoffset, 0), MAX (d_yoffset, 0));
+  if (dx < 0 || dy < 0)
+    gdk_window_queue_translation (window, MIN (dx, 0), MIN (dy, 0));
   
   XMoveResizeWindow (GDK_WINDOW_XDISPLAY (window),
 		     GDK_WINDOW_XID (window),
@@ -366,7 +361,7 @@ gdk_window_guffaw_scroll (GdkWindow    *window,
 		     impl->position_info.width, impl->position_info.height);
   
   if (impl->position_info.no_bg)
-    gdk_window_tmp_reset_bg (window);
+    _gdk_x11_window_tmp_reset_bg (window, FALSE);
   
   impl->position_info = new_info;
   
@@ -402,7 +397,7 @@ gdk_window_scroll (GdkWindow *window,
   GdkRegion *invalidate_region;
   GdkWindowImplX11 *impl;
   GdkWindowObject *obj;
-  GdkRectangle dest_rect;
+  GdkRectangle src_rect, dest_rect;
   
   g_return_if_fail (GDK_IS_WINDOW (window));
 
@@ -419,12 +414,23 @@ gdk_window_scroll (GdkWindow *window,
   if (obj->update_area)
     gdk_region_offset (obj->update_area, dx, dy);
   
-  invalidate_region = gdk_region_rectangle (&impl->position_info.clip_rect);
+  /* impl->position_info.clip_rect isn't meaningful for toplevels */
+  if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_CHILD)
+    src_rect = impl->position_info.clip_rect;
+  else
+    {
+      src_rect.x = 0;
+      src_rect.y = 0;
+      src_rect.width = impl->width;
+      src_rect.height = impl->height;
+    }
   
-  dest_rect = impl->position_info.clip_rect;
+  invalidate_region = gdk_region_rectangle (&src_rect);
+
+  dest_rect = src_rect;
   dest_rect.x += dx;
   dest_rect.y += dy;
-  gdk_rectangle_intersect (&dest_rect, &impl->position_info.clip_rect, &dest_rect);
+  gdk_rectangle_intersect (&dest_rect, &src_rect, &dest_rect);
 
   if (dest_rect.width > 0 && dest_rect.height > 0)
     {
@@ -542,7 +548,7 @@ _gdk_window_move_resize_child (GdkWindow *window,
 			 new_info.x, new_info.y, new_info.width, new_info.height);
       
       if (impl->position_info.no_bg)
-	gdk_window_tmp_reset_bg (window);
+	_gdk_x11_window_tmp_reset_bg (window, FALSE);
 
       if (!impl->position_info.mapped && new_info.mapped && GDK_WINDOW_IS_MAPPED (obj))
 	XMapWindow (GDK_WINDOW_XDISPLAY (window), GDK_WINDOW_XID (window));
@@ -588,7 +594,7 @@ _gdk_window_move_resize_child (GdkWindow *window,
 	}
 
       if (impl->position_info.no_bg)
-	gdk_window_tmp_reset_bg (window);
+	_gdk_x11_window_tmp_reset_bg (window, FALSE);
 
       if (!impl->position_info.mapped && new_info.mapped && GDK_WINDOW_IS_MAPPED (obj))
 	XMapWindow (GDK_WINDOW_XDISPLAY (window), GDK_WINDOW_XID (window));
@@ -848,7 +854,7 @@ gdk_window_postmove (GdkWindow          *window,
     XMapWindow (GDK_DRAWABLE_XDISPLAY (window), GDK_DRAWABLE_XID (window));
 
   if (impl->position_info.no_bg)
-    gdk_window_tmp_reset_bg (window);
+    _gdk_x11_window_tmp_reset_bg (window, FALSE);
 
   impl->position_info = new_info;
 
@@ -860,7 +866,7 @@ gdk_window_postmove (GdkWindow          *window,
     }
 }
 
-Bool
+static Bool
 expose_serial_predicate (Display *xdisplay,
 			 XEvent  *xev,
 			 XPointer arg)
@@ -903,7 +909,11 @@ queue_delete_link (GQueue *queue,
 static void
 queue_item_free (GdkWindowQueueItem *item)
 {
-  gdk_drawable_unref (item->window);
+  if (item->window)
+    {
+      g_object_remove_weak_pointer (G_OBJECT (item->window),
+				    (gpointer *)&(item->window));
+    }
   
   if (item->type == GDK_WINDOW_QUEUE_ANTIEXPOSE)
     gdk_region_destroy (item->u.antiexpose.area);
@@ -915,17 +925,19 @@ static void
 gdk_window_queue (GdkWindow          *window,
 		  GdkWindowQueueItem *item)
 {
-  if (!translate_queue)
-    translate_queue = g_queue_new ();
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (GDK_WINDOW_DISPLAY (window));
+  
+  if (!display_x11->translate_queue)
+    display_x11->translate_queue = g_queue_new ();
 
   /* Keep length of queue finite by, if it grows too long,
    * figuring out the latest relevant serial and discarding
    * irrelevant queue items.
    */
-  if (translate_queue->length >= 64 )
+  if (display_x11->translate_queue->length >= 64)
     {
       gulong serial = find_current_serial (GDK_WINDOW_XDISPLAY (window));
-      GList *tmp_list = translate_queue->head;
+      GList *tmp_list = display_x11->translate_queue->head;
       
       while (tmp_list)
 	{
@@ -934,7 +946,7 @@ gdk_window_queue (GdkWindow          *window,
 	  
 	  if (serial > item->serial)
 	    {
-	      queue_delete_link (translate_queue, tmp_list);
+	      queue_delete_link (display_x11->translate_queue, tmp_list);
 	      queue_item_free (item);
 	    }
 
@@ -948,9 +960,9 @@ gdk_window_queue (GdkWindow          *window,
    * discard anti-expose items. (We can't discard translate
    * items 
    */
-  if (translate_queue->length >= 64 )
+  if (display_x11->translate_queue->length >= 64)
     {
-      GList *tmp_list = translate_queue->head;
+      GList *tmp_list = display_x11->translate_queue->head;
       
       while (tmp_list)
 	{
@@ -959,20 +971,21 @@ gdk_window_queue (GdkWindow          *window,
 	  
 	  if (item->type == GDK_WINDOW_QUEUE_ANTIEXPOSE)
 	    {
-	      queue_delete_link (translate_queue, tmp_list);
+	      queue_delete_link (display_x11->translate_queue, tmp_list);
 	      queue_item_free (item);
 	    }
 
 	  tmp_list = next;
 	}
     }
-      
-  gdk_drawable_ref (window);
 
   item->window = window;
   item->serial = NextRequest (GDK_WINDOW_XDISPLAY (window));
   
-  g_queue_push_tail (translate_queue, item);
+  g_object_add_weak_pointer (G_OBJECT (window),
+			     (gpointer *)&(item->window));
+
+  g_queue_push_tail (display_x11->translate_queue, item);
 }
 
 static void
@@ -1009,12 +1022,12 @@ _gdk_window_process_expose (GdkWindow    *window,
   GdkWindowImplX11 *impl;
   GdkRegion *invalidate_region = gdk_region_rectangle (area);
   GdkRegion *clip_region;
-
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (GDK_WINDOW_DISPLAY (window));
   impl = GDK_WINDOW_IMPL_X11 (GDK_WINDOW_OBJECT (window)->impl);
       
-  if (translate_queue)
+  if (display_x11->translate_queue)
     {
-      GList *tmp_list = translate_queue->head;
+      GList *tmp_list = display_x11->translate_queue->head;
       
       while (tmp_list)
 	{
@@ -1033,7 +1046,8 @@ _gdk_window_process_expose (GdkWindow    *window,
 	    }
 	  else
 	    {
-	      queue_delete_link (translate_queue, translate_queue->head);
+	      queue_delete_link (display_x11->translate_queue, 
+				 display_x11->translate_queue->head);
 	      queue_item_free (item);
 	    }
 	}
@@ -1049,56 +1063,6 @@ _gdk_window_process_expose (GdkWindow    *window,
   gdk_region_destroy (clip_region);
 }
 
-static void
-gdk_window_tmp_unset_bg (GdkWindow *window)
-{
-  GdkWindowImplX11 *impl;
-  GdkWindowObject *obj;
-
-  obj = (GdkWindowObject *) window;
-  impl = GDK_WINDOW_IMPL_X11 (obj->impl);
-
-  impl->position_info.no_bg = TRUE;
-
-  if (obj->bg_pixmap != GDK_NO_BG)
-    XSetWindowBackgroundPixmap (GDK_DRAWABLE_XDISPLAY (window),
-				GDK_DRAWABLE_XID (window), None);
-}
-
-static void
-gdk_window_tmp_reset_bg (GdkWindow *window)
-{
-  GdkWindowImplX11 *impl;
-  GdkWindowObject *obj;
-
-  obj = (GdkWindowObject *) window;
-  impl = GDK_WINDOW_IMPL_X11 (obj->impl);
-
-  impl->position_info.no_bg = FALSE;
-
-  if (obj->bg_pixmap == GDK_NO_BG)
-    return;
-  
-  if (obj->bg_pixmap)
-    {
-      Pixmap xpixmap;
-
-      if (obj->bg_pixmap == GDK_PARENT_RELATIVE_BG)
-	xpixmap = ParentRelative;
-      else 
-	xpixmap = GDK_DRAWABLE_XID (obj->bg_pixmap);
-
-      XSetWindowBackgroundPixmap (GDK_DRAWABLE_XDISPLAY (window),
-				  GDK_DRAWABLE_XID (window), xpixmap);
-    }
-  else
-    {
-      XSetWindowBackground (GDK_DRAWABLE_XDISPLAY (window),
-			    GDK_DRAWABLE_XID (window),
-			    obj->bg_color.pixel);
-    }
-}
-
 static void
 gdk_window_clip_changed (GdkWindow *window, GdkRectangle *old_clip, GdkRectangle *new_clip)
 {
@@ -1131,7 +1095,7 @@ gdk_window_clip_changed (GdkWindow *window, GdkRectangle *old_clip, GdkRectangle
   gdk_region_subtract (new_clip_region, old_clip_region);
   if (!gdk_region_empty (new_clip_region))
     {
-      gdk_window_tmp_unset_bg (window);
+      _gdk_x11_window_tmp_unset_bg (window, FALSE);;
       gdk_window_invalidate_region (window, new_clip_region, FALSE);
     }
 
