@@ -1,7 +1,7 @@
 /* GTK - The GIMP Toolkit
  * Copyright (C) 2001 CodeFactory AB
  * Copyright (C) 2001, 2002 Anders Carlsson
- * Copyright (C) 2003, 2004 Matthias Clasen <mclasen@redhat.com>
+ * Copyright (C) 2003, 2004, 2005 Matthias Clasen <mclasen@redhat.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public 
@@ -115,6 +115,11 @@ static void                 gtk_about_dialog_set_property   (GObject
 							     guint               prop_id,
 							     const GValue       *value,
 							     GParamSpec         *pspec);
+static void                 gtk_about_dialog_style_set      (GtkWidget          *widget,
+			                                     GtkStyle           *previous_style);
+static void                 dialog_style_set                (GtkWidget          *widget,
+		                                             GtkStyle           *previous_style,
+		                                             gpointer            data);
 static void                 update_name_version             (GtkAboutDialog     *about);
 static GtkIconSet *         icon_set_new_from_pixbufs       (GList              *pixbufs);
 static void                 activate_url                    (GtkWidget          *widget,
@@ -153,6 +158,8 @@ static void                 display_credits_dialog          (GtkWidget
 							     gpointer            data);
 static void                 display_license_dialog          (GtkWidget          *button,
 							     gpointer            data);
+static void                 close_cb                        (GtkAboutDialog     *about);
+
 				 
 				 				       
 static GtkAboutDialogActivateLinkFunc activate_email_hook = NULL;
@@ -181,6 +188,8 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 
   object_class->finalize = gtk_about_dialog_finalize;
 
+  widget_class->style_set = gtk_about_dialog_style_set;
+
   /**
    * GtkAboutDialog:name:
    *
@@ -250,7 +259,8 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
    *
    * The license of the program. This string is displayed in a 
    * text view in a secondary dialog, therefore it is fine to use
-   * a long multi-paragraph text.
+   * a long multi-paragraph text. Note that the text is not wrapped
+   * in the text view, thus it must contain the intended linebreaks.
    *
    * Since: 2.6
    */  
@@ -424,6 +434,7 @@ gtk_about_dialog_init (GtkAboutDialog *about)
   priv->website = NULL;
   priv->website_label = NULL;
   priv->translator_credits = NULL;
+  priv->license = NULL;
   priv->authors = NULL;
   priv->documenters = NULL;
   priv->artists = NULL;
@@ -437,7 +448,6 @@ gtk_about_dialog_init (GtkAboutDialog *about)
   /* Widgets */
   gtk_widget_push_composite_child ();
   vbox = gtk_vbox_new (FALSE, 8);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
 
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about)->vbox), vbox, TRUE, TRUE, 0);
 
@@ -498,6 +508,13 @@ gtk_about_dialog_init (GtkAboutDialog *about)
   gtk_window_set_resizable (GTK_WINDOW (about), FALSE);
 
   gtk_widget_pop_composite_child ();
+
+  /* force defaults */
+  gtk_about_dialog_set_name (about, NULL);
+  gtk_about_dialog_set_logo (about, NULL);
+
+  /* Close dialog on user response */
+  g_signal_connect (about, "response", G_CALLBACK (close_cb), NULL);
 }
 
 static void
@@ -647,6 +664,37 @@ gtk_about_dialog_get_property (GObject    *object,
     }
 }
 
+static void
+dialog_style_set (GtkWidget *widget,
+		  GtkStyle *previous_style,
+		  gpointer data)
+{
+  GtkDialog *dialog;
+
+  dialog = GTK_DIALOG (widget);
+
+  /* Override the style properties with HIG-compliant spacings.  Ugh.
+   * http://developer.gnome.org/projects/gup/hig/1.0/layout.html#layout-dialogs
+   * http://developer.gnome.org/projects/gup/hig/1.0/windows.html#alert-spacing
+   */
+
+  gtk_container_set_border_width (GTK_CONTAINER (dialog->vbox), 12);
+  gtk_box_set_spacing (GTK_BOX (dialog->vbox), 12);
+
+  gtk_container_set_border_width (GTK_CONTAINER (dialog->action_area), 0);
+  gtk_box_set_spacing (GTK_BOX (dialog->action_area), 6);
+}
+
+static void
+gtk_about_dialog_style_set (GtkWidget *widget,
+			    GtkStyle  *previous_style)
+{
+  if (GTK_WIDGET_CLASS (gtk_about_dialog_parent_class)->style_set)
+    GTK_WIDGET_CLASS (gtk_about_dialog_parent_class)->style_set (widget, previous_style);
+
+  dialog_style_set (widget, previous_style, NULL);
+}
+
 /**
  * gtk_about_dialog_get_name:
  * @about: a #GtkAboutDialog
@@ -1439,7 +1487,7 @@ gtk_about_dialog_set_logo (GtkAboutDialog *about,
  * Returns the icon name displayed as logo in the about dialog.
  * 
  * Return value: the icon name displayed as logo. The string is
- *   owned by the about dialog. If you want to keep a reference
+ *   owned by the dialog. If you want to keep a reference
  *   to it, you have to call g_strdup() on it.
  *
  * Since: 2.6
@@ -1448,7 +1496,7 @@ G_CONST_RETURN gchar *
 gtk_about_dialog_get_logo_icon_name (GtkAboutDialog *about)
 {
   GtkAboutDialogPrivate *priv;
-  const gchar *icon_name;
+  const gchar *icon_name = NULL;
   
   g_return_val_if_fail (GTK_IS_ABOUT_DIALOG (about), NULL);
 
@@ -1531,6 +1579,27 @@ set_link_button_text (GtkWidget *about,
   g_free (link);
 }
 
+static gboolean
+link_button_enter (GtkWidget        *widget,
+		   GdkEventCrossing *event,
+		   GtkAboutDialog   *about)
+{
+  GtkAboutDialogPrivate *priv = (GtkAboutDialogPrivate *)about->private_data;
+  gdk_window_set_cursor (widget->window, priv->hand_cursor);
+
+  return FALSE;
+}
+
+static gboolean
+link_button_leave (GtkWidget        *widget,
+		   GdkEventCrossing *event,
+		   GtkAboutDialog   *about)
+{
+  gdk_window_set_cursor (widget->window, NULL);
+
+  return FALSE;
+}
+
 static GtkWidget *
 create_link_button (GtkWidget *about,
 		    gchar     *text,
@@ -1547,7 +1616,11 @@ create_link_button (GtkWidget *about,
   g_object_set_data_full (G_OBJECT (button), "url", g_strdup (url), g_free);
   set_link_button_text (about, button, text);
   
-  g_signal_connect (G_OBJECT (button), "clicked", callback, data);
+  g_signal_connect (button, "clicked", callback, data);
+  g_signal_connect (button, "enter_notify_event",
+		    G_CALLBACK (link_button_enter), data);
+  g_signal_connect (button, "leave_notify_event",
+		    G_CALLBACK (link_button_leave), data);
 
   return button;
 }
@@ -1763,19 +1836,22 @@ add_credits_page (GtkAboutDialog *about,
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
   gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (view), FALSE);
   gtk_text_view_set_editable (GTK_TEXT_VIEW (view), FALSE);
+
   gtk_text_view_set_left_margin (GTK_TEXT_VIEW (view), 8);
   gtk_text_view_set_right_margin (GTK_TEXT_VIEW (view), 8);
 
-  g_signal_connect (G_OBJECT (view), "key-press-event",
+  g_signal_connect (view, "key-press-event",
                     G_CALLBACK (credits_key_press_event), about);
-  g_signal_connect (G_OBJECT (view), "event-after",
+  g_signal_connect (view, "event-after",
                     G_CALLBACK (credits_event_after), about);
-  g_signal_connect (G_OBJECT (view), "motion-notify-event", 
+  g_signal_connect (view, "motion-notify-event", 
                     G_CALLBACK (credits_motion_notify_event), about);
-  g_signal_connect (G_OBJECT (view), "visibility-notify-event", 
+  g_signal_connect (view, "visibility-notify-event", 
                     G_CALLBACK (credits_visibility_notify_event), about);
 
   sw = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
+  				       GTK_SHADOW_IN);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 				  GTK_POLICY_AUTOMATIC,
 				  GTK_POLICY_AUTOMATIC);
@@ -1891,9 +1967,10 @@ display_credits_dialog (GtkWidget *button,
   g_signal_connect (dialog, "destroy",
 		    G_CALLBACK (gtk_widget_destroyed),
 		    &(priv->credits_dialog));
+  g_signal_connect (dialog, "style_set",
+		    G_CALLBACK (dialog_style_set), NULL);
 
   notebook = gtk_notebook_new ();
-  gtk_container_set_border_width (GTK_CONTAINER (notebook), 8);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), notebook, TRUE, TRUE, 0);
 
   if (priv->authors != NULL) 
@@ -1921,6 +1998,14 @@ display_credits_dialog (GtkWidget *button,
   gtk_widget_show_all (dialog);
 }
 
+static void
+set_policy (GtkWidget *sw)
+{
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+				  GTK_POLICY_AUTOMATIC,
+				  GTK_POLICY_AUTOMATIC);  
+}
+
 static void
 display_license_dialog (GtkWidget *button, 
 			gpointer   data)
@@ -1953,14 +2038,16 @@ display_license_dialog (GtkWidget *button,
   g_signal_connect (dialog, "destroy",
 		    G_CALLBACK (gtk_widget_destroyed),
 		    &(priv->license_dialog));
+  g_signal_connect (dialog, "style_set",
+		    G_CALLBACK (dialog_style_set), NULL);
 
   sw = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
 				       GTK_SHADOW_IN);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-				  GTK_POLICY_AUTOMATIC,
+				  GTK_POLICY_NEVER,
 				  GTK_POLICY_AUTOMATIC);
-  gtk_container_set_border_width (GTK_CONTAINER (sw), 8);
+  g_signal_connect (sw, "map", G_CALLBACK (set_policy), NULL);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), sw, TRUE, TRUE, 0);
 
   view = gtk_text_view_new ();
@@ -1969,6 +2056,7 @@ display_license_dialog (GtkWidget *button,
 
   gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (view), FALSE);
   gtk_text_view_set_editable (GTK_TEXT_VIEW (view), FALSE);
+
   gtk_text_view_set_left_margin (GTK_TEXT_VIEW (view), 8);
   gtk_text_view_set_right_margin (GTK_TEXT_VIEW (view), 8);
 
@@ -2010,17 +2098,7 @@ close_cb (GtkAboutDialog *about)
 GtkWidget *
 gtk_about_dialog_new (void)
 {
-  GtkAboutDialog *dialog = GTK_ABOUT_DIALOG (g_object_new (GTK_TYPE_ABOUT_DIALOG, NULL));
-
-  /* force defaults */
-  gtk_about_dialog_set_name (dialog, NULL);
-  gtk_about_dialog_set_logo (dialog, NULL);
-
-  /* Close dialog on user response */
-  g_signal_connect (G_OBJECT (dialog),
-                    "response",
-                    G_CALLBACK (close_cb),
-                    NULL);
+  GtkAboutDialog *dialog = g_object_new (GTK_TYPE_ABOUT_DIALOG, NULL);
 
   return GTK_WIDGET (dialog);
 }
@@ -2046,7 +2124,7 @@ gtk_about_dialog_set_email_hook (GtkAboutDialogActivateLinkFunc func,
   GtkAboutDialogActivateLinkFunc old;
 
   if (activate_email_hook_destroy != NULL)
-    (* activate_email_hook_destroy) (activate_url_hook_data);
+    (* activate_email_hook_destroy) (activate_email_hook_data);
 
   old = activate_email_hook;
 
@@ -2078,7 +2156,7 @@ gtk_about_dialog_set_url_hook (GtkAboutDialogActivateLinkFunc func,
   GtkAboutDialogActivateLinkFunc old;
 
   if (activate_url_hook_destroy != NULL)
-    (* activate_url_hook_destroy) (activate_email_hook_data);
+    (* activate_url_hook_destroy) (activate_url_hook_data);
 
   old = activate_url_hook;
 
@@ -2119,7 +2197,7 @@ gtk_show_about_dialog (GtkWindow   *parent,
     {
       dialog = gtk_about_dialog_new ();
 
-      g_object_ref (G_OBJECT (dialog));
+      g_object_ref (dialog);
       gtk_object_sink (GTK_OBJECT (dialog));
 
       g_signal_connect (dialog, "delete_event", G_CALLBACK (gtk_widget_hide_on_delete), NULL);
