@@ -633,7 +633,7 @@ gtk_im_context_xim_new (void)
   GtkIMContextXIM *result;
   const gchar *charset;
 
-  result = GTK_IM_CONTEXT_XIM (g_object_new (GTK_TYPE_IM_CONTEXT_XIM, NULL));
+  result = g_object_new (GTK_TYPE_IM_CONTEXT_XIM, NULL);
 
   result->locale = g_strdup (setlocale (LC_CTYPE, NULL));
   
@@ -810,7 +810,7 @@ gtk_im_context_xim_set_cursor_location (GtkIMContext *context,
 
   preedit_attr = XVaCreateNestedList (0,
 				      XNSpotLocation, &spot,
-				      0);
+				      NULL);
   XSetICValues (ic,
 		XNPreeditAttributes, preedit_attr,
 		NULL);
@@ -857,7 +857,7 @@ gtk_im_context_xim_reset (GtkIMContext *context)
 
   preedit_attr = XVaCreateNestedList(0,
                                      XNPreeditState, &preedit_state,
-                                     0);
+                                     NULL);
   if (!XGetICValues(ic,
                     XNPreeditAttributes, preedit_attr,
                     NULL))
@@ -869,7 +869,7 @@ gtk_im_context_xim_reset (GtkIMContext *context)
 
   preedit_attr = XVaCreateNestedList(0,
                                      XNPreeditState, preedit_state,
-                                     0);
+                                     NULL);
   if (have_preedit_state)
     XSetICValues(ic,
 		 XNPreeditAttributes, preedit_attr,
